from label_studio_converter import Converter
import os
import pytest
import tempfile
import shutil

BASE_DIR = os.path.dirname(__file__)
TEST_DATA_PATH = os.path.join(BASE_DIR, "data", "test_export_yolo")
INPUT_JSON_PATH = os.path.join(BASE_DIR, TEST_DATA_PATH, "data.json")
LABEL_CONFIG_PATH = os.path.join(BASE_DIR, TEST_DATA_PATH, "label_config.xml")
INPUT_JSON_PATH_POLYGONS = os.path.join(BASE_DIR, TEST_DATA_PATH, "data_polygons.json")
LABEL_CONFIG_PATH_POLYGONS = os.path.join(
    BASE_DIR, TEST_DATA_PATH, "label_config_polygons.xml"
)


def check_equal_list_of_strings(list1, list2):
    # Check that both lists are not empty
    if not list1 or not list2:
        return False

    list1.sort()
    list2.sort()

    # Check that the lists have the same length
    if len(list1) != len(list2):
        return False

    # Check that the elements of the lists are equal
    for i in range(len(list1)):
        if list1[i] != list2[i]:
            return False

    return True


def get_os_walk(root_path):
    list_file_paths = []
    for root, dirs, files in os.walk(root_path):
        for f in files:
            list_file_paths += [os.path.join(root, f)]
    return list_file_paths


@pytest.fixture
def create_temp_folder():
    # Create a temporary folder
    temp_dir = tempfile.mkdtemp()

    # Yield the temporary folder
    yield temp_dir

    # Remove the temporary folder after the test
    shutil.rmtree(temp_dir)


def test_convert_to_yolo(create_temp_folder):
    """Check converstion label_studio json exported file to yolo with multiple labelers"""

    # Generates a temporary folder and return the absolute path
    # The temporary folder contains all the data generate by the following function
    # For debugging replace create_temp_folder with "./tmp"
    tmp_folder = create_temp_folder

    output_dir = tmp_folder
    output_image_dir = os.path.join(output_dir, "tmp_image")
    output_label_dir = os.path.join(output_dir, "tmp_label")
    project_dir = "."

    converter = Converter(LABEL_CONFIG_PATH, project_dir)
    converter.convert_to_yolo(
        INPUT_JSON_PATH,
        output_dir,
        output_image_dir=output_image_dir,
        output_label_dir=output_label_dir,
        is_dir=False,
        split_labelers=True,
    )

    abs_path_label_dir = os.path.abspath(output_label_dir)
    expected_paths = [
        os.path.join(abs_path_label_dir, "1", "image1.txt"),
        os.path.join(abs_path_label_dir, "1", "image2.txt"),
        os.path.join(abs_path_label_dir, "2", "image1.txt"),
    ]
    generated_paths = get_os_walk(abs_path_label_dir)
    # Check all files and subfolders have been generated.
    assert check_equal_list_of_strings(
        expected_paths, generated_paths
    ), f"Generated file: \n  {generated_paths} \n does not match expected ones: \n {expected_paths}"
    # Check all the annotations have been converted to yolo
    for file in expected_paths:
        with open(file) as f:
            lines = f.readlines()
            assert (
                len(lines) == 2
            ), f"Expect different number of annotations in file {file}."


def test_convert_polygons_to_yolo(create_temp_folder):
    """Check converstion label_studio json exported file to yolo with polygons"""

    # Generates a temporary folder and return the absolute path
    # The temporary folder contains all the data generate by the following function
    # For debugging replace create_temp_folder with "./tmp"
    tmp_folder = create_temp_folder

    output_dir = tmp_folder
    output_image_dir = os.path.join(output_dir, "tmp_image")
    output_label_dir = os.path.join(output_dir, "tmp_label")
    project_dir = "."

    converter = Converter(LABEL_CONFIG_PATH_POLYGONS, project_dir)
    converter.convert_to_yolo(
        INPUT_JSON_PATH_POLYGONS,
        output_dir,
        output_image_dir=output_image_dir,
        output_label_dir=output_label_dir,
        is_dir=False,
        split_labelers=False,
    )

    abs_path_label_dir = os.path.abspath(output_label_dir)
    expected_paths = [os.path.join(abs_path_label_dir, "image2.txt")]
    generated_paths = get_os_walk(abs_path_label_dir)
    # Check all files and subfolders have been generated.
    assert check_equal_list_of_strings(
        expected_paths, generated_paths
    ), f"Generated file: \n  {generated_paths} \n does not match expected ones: \n {expected_paths}"
    # Check all the annotations have been converted to yolo
    for file in expected_paths:
        with open(file) as f:
            lines = f.readlines()
            assert (
                len(lines) == 1
            ), f"Expect different number of annotations in file {file}."



import os
import io
import logging
import argparse

from label_studio_converter.converter import Converter, Format, FormatNotSupportedError
from label_studio_converter.exports.csv import ExportToCSV
from label_studio_converter.utils import ExpandFullPath
from label_studio_converter.imports import yolo as import_yolo, coco as import_coco

logging.basicConfig(level=logging.INFO)


def get_export_args(parser):
    parser.add_argument(
        '-i',
        '--input',
        dest='input',
        required=True,
        help='Directory or JSON file with annotations (e.g. "/<project_path>/annotations")',
        action=ExpandFullPath,
    )
    parser.add_argument(
        '-c',
        '--config',
        dest='config',
        help='Project config (e.g. "/<project_path>/config.xml")',
        action=ExpandFullPath,
    )
    parser.add_argument(
        '-o',
        '--output',
        dest='output',
        help='Output file or directory (will be created if not exists)',
        default=os.path.join(os.path.dirname(__file__), 'output'),
        action=ExpandFullPath,
    )
    parser.add_argument(
        '-f',
        '--format',
        dest='format',
        metavar='FORMAT',
        help='Output format: ' + ', '.join(f.name for f in Format),
        type=Format.from_string,
        choices=list(Format),
        default=Format.JSON,
    )
    parser.add_argument(
        '--csv-separator',
        dest='csv_separator',
        help='Separator used in CSV format',
        default=',',
    )
    parser.add_argument(
        '--csv-no-header',
        dest='csv_no_header',
        help='Whether to omit header in CSV output file',
        action='store_true',
    )
    parser.add_argument(
        '--image-dir',
        dest='image_dir',
        help='In case of image outputs (COCO, VOC, ...), specifies output image directory where downloaded images will '
        'be stored. (If not specified, local image paths left untouched)',
    )
    parser.add_argument(
        '--project-dir',
        dest='project_dir',
        default=None,
        help='Label Studio project directory path',
    )
    parser.add_argument(
        '--heartex-format',
        dest='heartex_format',
        action='store_true',
        default=True,
        help='Set this flag if your annotations are in one JSON file instead of multiple JSON files from directory',
    )


def get_all_args():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(dest='command', help='Available commands')
    subparsers.required = False

    # Export
    parser_export = subparsers.add_parser(
        'export',
        help='Converter from Label Studio JSON annotations to external formats',
    )
    get_export_args(parser_export)

    # Import
    parser_import = subparsers.add_parser(
        'import',
        help="Converter from external formats to Label Studio JSON annotations",
    )
    import_format = parser_import.add_subparsers(dest='import_format')
    import_yolo.add_parser(import_format)
    import_coco.add_parser(import_format)

    return parser.parse_args()


def export(args):
    c = Converter(args.config, project_dir=args.project_dir)

    if args.format == Format.JSON:
        c.convert_to_json(args.input, args.output)
    elif args.format == Format.CSV:
        header = not args.csv_no_header
        sep = args.csv_separator
        c.convert_to_csv(
            args.input,
            args.output,
            sep=sep,
            header=header,
            is_dir=not args.heartex_format,
        )
    elif args.format == Format.CSV_OLD:
        header = not args.csv_no_header
        sep = args.csv_separator
        ExportToCSV(args.input).to_file(
            args.output, sep=sep, header=header, index=False
        )
    elif args.format == Format.TSV:
        header = not args.csv_no_header
        sep = '\t'
        c.convert_to_csv(
            args.input,
            args.output,
            sep=sep,
            header=header,
            is_dir=not args.heartex_format,
        )
    elif args.format == Format.CONLL2003:
        c.convert_to_conll2003(args.input, args.output, is_dir=not args.heartex_format)
    elif args.format == Format.COCO:
        c.convert_to_coco(
            args.input,
            args.output,
            output_image_dir=args.image_dir,
            is_dir=not args.heartex_format,
        )
    elif args.format == Format.VOC:
        c.convert_to_voc(
            args.input,
            args.output,
            output_image_dir=args.image_dir,
            is_dir=not args.heartex_format,
        )
    elif args.format == Format.YOLO:
        c.convert_to_yolo(args.input, args.output, is_dir=not args.heartex_format)
    else:
        raise FormatNotSupportedError()


def imports(args):
    if args.import_format == 'yolo':
        import_yolo.convert_yolo_to_ls(
            input_dir=args.input,
            out_file=args.output,
            to_name=args.to_name,
            from_name=args.from_name,
            out_type=args.out_type,
            image_root_url=args.image_root_url,
            image_ext=args.image_ext,
        )

    elif args.import_format == 'coco':
        import_coco.convert_coco_to_ls(
            input_file=args.input,
            out_file=args.output,
            to_name=args.to_name,
            from_name=args.from_name,
            out_type=args.out_type,
            image_root_url=args.image_root_url,
            point_width=args.point_width,
        )
    else:
        raise FormatNotSupportedError()


def main():
    args = get_all_args()
    if args.command == 'export':
        export(args)
    elif args.command == 'import':
        imports(args)
    else:
        print('Please, use "import" or "export" or "-h" command')


if __name__ == "__main__":
    main()



import json
import os
import glob
import lxml.etree as ET

from label_studio_converter.imports import yolo as import_yolo



def test_base_import_yolo():
    """Tests generated config and json files for yolo imports
    test_import_yolo_data folder assumes only images in the 'images' folder
    with corresponding labels existing in the 'labes' dir and a 'classes.txt' present.
    (currently 7 images -> 3 png, 2 jpg and 2 jpeg files)
    """
    input_data_dir = os.path.join(os.path.abspath(os.path.dirname(__file__)),'data','test_import_yolo_data')
    out_json_file = os.path.join('/tmp','lsc-pytest','yolo_exp_test.json')

    image_ext = '.jpg,.jpeg,.png' #comma seperated string of extns.

    import_yolo.convert_yolo_to_ls(
        input_dir=input_data_dir,
        out_file=out_json_file,
        image_ext=image_ext
    )

    #'yolo_exp_test.label_config.xml' and 'yolo_exp_test.json' must be generated.
    out_config_file = os.path.join('/tmp','lsc-pytest','yolo_exp_test.label_config.xml')
    assert os.path.exists(out_config_file) and os.path.exists(out_json_file), "> import failed! files not generated."

    #provided labels from classes.txt
    with open(os.path.join(input_data_dir,'classes.txt'), 'r') as f:
        labels = f.read()[:-1].split('\n') #[:-1] since last line in classes.txt is empty by convention

    #generated labels from config xml
    label_element = ET.parse(out_config_file).getroot()[2]
    lables_generated = [x.attrib['value'] for x in label_element.getchildren()]
    assert set(labels) == set(lables_generated), "> generated class labels do not match original labels"

    #total image files in the input folder
    img_files = glob.glob(os.path.join(input_data_dir,'images','*'))

    with open(out_json_file, 'r') as f:
        ls_data = json.loads(f.read())

    assert len(ls_data) == len(img_files), "some file imports did not succeed!"


def test_base_import_yolo_with_img_dims():
    """Tests generated config and json files for yolo imports while importing unique image dims
    """
    input_data_dir = os.path.join(os.path.abspath(os.path.dirname(__file__)),'data','test_import_yolo_data_unif_dims')
    out_json_file = os.path.join('/tmp','lsc-pytest','yolo_exp_test.json')

    image_ext = '.jpg,.jpeg,.png' # comma seperated string of extns.
    img_dims = (640, 329)  # known width and height of dataset

    import_yolo.convert_yolo_to_ls(
        input_dir=input_data_dir,
        out_file=out_json_file,
        image_ext=image_ext
    )

    # 'yolo_exp_test.label_config.xml' and 'yolo_exp_test.json' must be generated.
    out_config_file = os.path.join('/tmp','lsc-pytest','yolo_exp_test.label_config.xml')
    assert os.path.exists(out_config_file) and os.path.exists(out_json_file), "> import failed! files not generated."

    # provided labels from classes.txt
    with open(os.path.join(input_data_dir, 'classes.txt'), 'r') as f:
        labels = f.read()[:-1].split(
            '\n'
        )  # [:-1] since last line in classes.txt is empty by convention

    # generated labels from config xml
    label_element = ET.parse(out_config_file).getroot()[2]
    lables_generated = [x.attrib['value'] for x in label_element.getchildren()]
    assert set(labels) == set(
        lables_generated
    ), "> generated class labels do not match original labels"

    #total image files in the input folder
    img_files = glob.glob(os.path.join(input_data_dir,'images','*'))

    with open(out_json_file, 'r') as f:
        ls_data = json.loads(f.read())

    assert len(ls_data) == len(img_files), "some file imports did not succeed!"


if __name__ == '__main__':
    test_base_import_yolo()
    test_base_import_yolo_with_img_dims()
